@@ -46,6 +46,15 @@ def _len_epoch(input_shape, data_shape, target=0.01, delta = 0.005, df=2, max_le
     return t, n
 
 
+def load_data(x, data_type='pngseq', dtype='uint8'):
+    if isinstance(x, str):  # path to a single volume
+        return read_volume(x, type=data_type, dtype=dtype)
+    elif isinstance(x, np.ndarray):  # data already loaded
+        return x
+    else:
+        raise ValueError('Datasets require data in str or np.ndarray format, received %s' % type(x))
+
+
 def pad2multiple(x, input_shape, value=0):
 
     pad_width = []
@@ -136,9 +145,13 @@ class VolumeDataset(data.Dataset):
     """
     Dataset for volumes
 
-    :param data: path to the dataset or a 3D volume that has already been loaded
+    :param data: input data, possible formats
+            - path to the dataset (string)
+            - preloaded 3D volume (numpy array)
+            - list of paths to multiple datasets (list of strings)
+            - list of preloaded 3D volumes (list of numpy arrays)
     :param input_shape: 3-tuple that specifies the input shape for sampling
-    :param optional scaling: tuple used for rescaling the data, or None
+    :param optional scaling: 3-tuple used for rescaling the data, or a list of 3-tuples in case of multiple datasets
     :param optional len_epoch: number of iterations for one epoch
     :param optional type: type of the volume file (tif2d, tif3d, tifseq, hdf5, png or pngseq)
     :param optional in_channels: amount of subsequent slices to be sampled (only for 2D sampling)
@@ -146,20 +159,30 @@ class VolumeDataset(data.Dataset):
     :param optional batch_size: size of the sampling batch
     :param optional dtype: type of the data (typically uint8)
     :param optional norm_type: type of the normalization (unit, z or minmax)
-    :param optional range_split: range of slices (start, stop) to select (normalized between 0 and 1)
-    :param optional range_dir: orientation of the slicing
+    :param optional range_split: range of slices (start, stop) to select (normalized between 0 and 1), or a list of
+                                 2-tuples in case of multiple datasets
+    :param optional range_dir: orientation of the slicing or a list of orientations in case of multiple datasets
+    :param optional resolution: list of 3-tuples specifying the pixel resolution of the data
+    :param optional match_resolution_to: match the resolution of all data to a specific dataset
+    :param optional sampling_type: type of sampling in case of multiple datasets
+            - joint: the dataset will generate random samples in each dataset and return all of them
+            - single: the dataset will generate a random sample from a randomly selected dataset and return that
     """
 
-    def __init__(self, data, input_shape, scaling=None, len_epoch=None, type='tif3d', in_channels=1,
-                 orientations=(0,), batch_size=1, dtype='uint8', norm_type='unit', range_split=None, range_dir=None):
-        if isinstance(data, str):
-            self.data_path = data
-            # load the data
-            self.data = read_volume(data, type=type, dtype=dtype)
+    def __init__(self, data, input_shape, scaling=None, len_epoch=None, type='tif3d', in_channels=1, orientations=(0,),
+                 batch_size=1, dtype='uint8', norm_type='unit', range_split=None, range_dir=None, resolution=None,
+                 match_resolution_to=None, sampling_type='joint'):
+
+        if isinstance(data, str) or isinstance(data, np.ndarray):
+            self.data = [load_data(data, data_type=type, dtype=dtype)]
+        elif isinstance(data, list) or isinstance(data, tuple):  # list of data
+            self.data = []
+            for data_i in data:
+                self.data.append(load_data(data_i, data_type=type, dtype=dtype))
         else:
-            self.data = data
+            raise ValueError('LabeledVolumeDataset requires data in str, np.ndarray or list format')
         self.input_shape = input_shape
-        self.scaling = float(scaling) if scaling is not None else scaling
+        self.scaling = scaling
         self.len_epoch = int(len_epoch) if len_epoch is not None else len_epoch
         self.in_channels = int(in_channels)
         if isinstance(orientations, str):
@@ -171,23 +194,46 @@ class VolumeDataset(data.Dataset):
         self.norm_type = norm_type
         self.range_split = range_split
         self.range_dir = range_dir
+        self.resolution = resolution
+        self.match_resolution_to = match_resolution_to
+        self.sampling_type = sampling_type
+        self.warned = False
 
         # select a subset of slices of the data
-        self.data = slice_subset(self.data, range_split, range_dir)
+        for i in range(len(self.data)):
+            if isinstance(range_dir, list) or isinstance(range_dir, tuple):
+                self.data[i] = slice_subset(self.data[i], range_split[i], range_dir[i])
+            else:
+                self.data[i] = slice_subset(self.data[i], range_split, range_dir)
 
         # compute length epoch if necessary
         if len_epoch is None or len_epoch < 0:
             print_frm('Epoch length not set... estimating full coverage value')
-            t, self.len_epoch = _len_epoch(self.input_shape, self.data.shape)
-            print_frm('Epoch length automatically set to %d, this covers %.2f%% of the data on average'
+            if self.sampling_type == 'single':
+                tt, tle = 0, 0
+                for d in self.data:
+                    t, le = _len_epoch(self.input_shape, d.shape)
+                    tt += t
+                    tle += le
+                t, self.len_epoch = tt / len(self.data), tle
+            else:
+                i_max = int(np.argmax([d.size for d in self.data]))
+                t, self.len_epoch = _len_epoch(self.input_shape, self.data[i_max].shape)
+            print_frm('Epoch length automatically set to %d, this covers %.2f%% of the largest data on average'
                       % (self.len_epoch, (1-t)*100))
 
         # rescale the dataset if necessary
-        if scaling is not None:
-            target_size = np.asarray(np.multiply(self.data.shape, scaling), dtype=int)
-            self.data = \
-                F.interpolate(torch.Tensor(self.data[np.newaxis, np.newaxis, ...]), size=tuple(target_size),
-                              mode='area')[0, 0, ...].numpy()
+        for i in range(len(self.data)):
+            if (self.resolution is not None and self.match_resolution_to is not None) or self.scaling is not None:
+                if self.resolution is not None and self.match_resolution_to is not None:
+                    scale_factor = np.divide(self.data[self.match_resolution_to].shape, self.data[i].shape)
+                else:
+                    if isinstance(self.scaling, list):
+                        scale_factor = self.scaling[i]
+                    else:
+                        scale_factor = self.scaling
+                data_it = torch.Tensor(self.data[i]).unsqueeze(0).unsqueeze(0)
+                self.data[i] = F.interpolate(data_it, scale_factor=scale_factor, mode='bilinear')[0, 0, ...].numpy()
 
     def __getitem__(self, i):
         pass
@@ -195,12 +241,6 @@ class VolumeDataset(data.Dataset):
     def __len__(self):
         return self.len_epoch
 
-    def _get_stats(self):
-        mu = np.mean(self.data)
-        std = np.std(self.data)
-
-        return mu, std
-
     def _select_orientation(self):
         self.orientation = np.random.choice(self.orientations)
 
@@ -209,29 +249,40 @@ class SlidingWindowDataset(data.Dataset):
     """
     Dataset for sliding window over volumes
 
-    :param data: path to the dataset or a 3D volume that has already been loaded
+    :param data: input data, possible formats
+            - path to the dataset (string)
+            - preloaded 3D volume (numpy array)
+            - list of paths to multiple datasets (list of strings)
+            - list of preloaded 3D volumes (list of numpy arrays)
     :param input_shape: 3-tuple that specifies the input shape for sampling
-    :param optional scaling: tuple used for rescaling the data, or None
+    :param optional scaling: 3-tuple used for rescaling the data, or a list of 3-tuples in case of multiple datasets
     :param optional type: type of the volume file (tif2d, tif3d, tifseq, hdf5, png or pngseq)
     :param optional in_channels: amount of subsequent slices to be sampled (only for 2D sampling)
     :param optional orientations: list of orientations for sampling
     :param optional batch_size: size of the sampling batch
     :param optional dtype: type of the data (typically uint8)
     :param optional norm_type: type of the normalization (unit, z or minmax)
-    :param optional range_split: range of slices (start, stop) to select (normalized between 0 and 1)
-    :param optional range_dir: orientation of the slicing
+    :param optional range_split: range of slices (start, stop) to select (normalized between 0 and 1), or a list of
+                                 2-tuples in case of multiple datasets
+    :param optional range_dir: orientation of the slicing or a list of orientations in case of multiple datasets
+    :param optional resolution: list of 3-tuples specifying the pixel resolution of the data
+    :param optional match_resolution_to: match the resolution of all data to a specific dataset
     """
 
     def __init__(self, data, input_shape, scaling=None, type='tif3d', in_channels=1, orientations=(0,), batch_size=1,
-                 dtype='uint8', norm_type='unit', range_split=None, range_dir=None):
-        if isinstance(data, str):
-            self.data_path = data
-            # load the data
-            self.data = read_volume(data, type=type, dtype=dtype)
+                 dtype='uint8', norm_type='unit', range_split=None, range_dir=None, resolution=None,
+                 match_resolution_to=None):
+
+        if isinstance(data, str) or isinstance(data, np.ndarray):
+            self.data = [load_data(data, data_type=type, dtype=dtype)]
+        elif isinstance(data, list) or isinstance(data, tuple):  # list of data
+            self.data = []
+            for data_i in data:
+                self.data.append(load_data(data_i, data_type=type, dtype=dtype))
         else:
-            self.data = data
+            raise ValueError('SlidingWindowDataset requires data in str, np.ndarray or list format')
         self.input_shape = input_shape
-        self.scaling = float(scaling) if scaling is not None else scaling
+        self.scaling = scaling
         self.in_channels = int(in_channels)
         if isinstance(orientations, str):
             self.orientations = [int(item) for item in orientations.split(',')]
@@ -243,114 +294,45 @@ class SlidingWindowDataset(data.Dataset):
         self.norm_type = norm_type
         self.range_split = range_split
         self.range_dir = range_dir
+        self.resolution = resolution
+        self.match_resolution_to = match_resolution_to
+        self.warned = False
 
         # select a subset of slices of the data
-        self.data = slice_subset(self.data, range_split, range_dir)
+        for i in range(len(self.data)):
+            if isinstance(range_dir, list) or isinstance(range_dir, tuple):
+                self.data[i] = slice_subset(self.data[i], range_split[i], range_dir[i])
+            else:
+                self.data[i] = slice_subset(self.data[i], range_split, range_dir)
 
         # rescale the dataset if necessary
-        if scaling is not None:
-            target_size = np.asarray(np.multiply(self.data.shape, scaling), dtype=int)
-            self.data = \
-                F.interpolate(torch.Tensor(self.data[np.newaxis, np.newaxis, ...]), size=tuple(target_size),
-                              mode='area')[0, 0, ...].numpy()
+        for i in range(len(self.data)):
+            if (self.resolution is not None and self.match_resolution_to is not None) or self.scaling is not None:
+                if self.resolution is not None and self.match_resolution_to is not None:
+                    scale_factor = np.divide(self.data[self.match_resolution_to].shape, self.data[i].shape)
+                else:
+                    if isinstance(self.scaling, list):
+                        scale_factor = self.scaling[i]
+                    else:
+                        scale_factor = self.scaling
+                data_it = torch.Tensor(self.data[i]).unsqueeze(0).unsqueeze(0)
+                self.data[i] = F.interpolate(data_it, scale_factor=scale_factor, mode='bilinear')[0, 0, ...].numpy()
 
         # pad data so that the dimensions are a multiple of the inputs shapes
-        self.data = pad2multiple(self.data, input_shape, value=0)
+        self.data = [pad2multiple(d, input_shape, value=0) for d in self.data]
 
         # compute size of the data
-        sz = 1
-        self.n_samples_dim = []
-        for d in range(3):
-            self.n_samples_dim.append(self.data.shape[d] // self.input_shape[d])
-            sz *= self.n_samples_dim[-1]
-        self.n_samples = sz
+        self.n_samples_dim = np.zeros((len(self.data), 3), dtype=int)
+        for i, data_i in enumerate(self.data):
+            for d in range(3):
+                self.n_samples_dim[i, d] = data_i.shape[d] // self.input_shape[d]
+        self.n_samples = self.n_samples_dim.prod(axis=1).sum()
 
         # pad data so that the dimensions are a multiple of the inputs shapes
-        self.data = pad_channels(self.data, in_channels=in_channels, orientations=self.orientations)
+        self.data = [pad_channels(d, in_channels=in_channels, orientations=self.orientations) for d in self.data]
 
     def __getitem__(self, i):
         pass
 
     def __len__(self):
         return self.n_samples
-
-    def _get_stats(self):
-        mu = np.mean(self.data)
-        std = np.std(self.data)
-
-        return mu, std
-
-
-class MultiVolumeDataset(data.Dataset):
-    """
-    Dataset for multiple volumes
-
-    :param data_path: list of paths to the datasets
-    :param input_shape: 3-tuple that specifies the input shape for sampling
-    :param optional scaling: tuple used for rescaling the data, or None
-    :param optional len_epoch: number of iterations for one epoch
-    :param optional types: list of types of the volume files (tif2d, tif3d, tifseq, hdf5, png or pngseq)
-    :param optional in_channels: amount of subsequent slices to be sampled (only for 2D sampling)
-    :param optional sampling_mode: allow for uniform balance in sampling or not ("uniform" or "random")
-    :param optional orientations: list of orientations for sampling
-    :param optional batch_size: size of the sampling batch
-    :param optional dtype: type of the data (typically uint8)
-    :param optional norm_type: type of the normalization (unit, z or minmax)
-    """
-
-    def __init__(self, data_path, input_shape, scaling=None, len_epoch=1000, types=['tif3d'], sampling_mode='uniform',
-                 in_channels=1, orientations=(0,), batch_size=1, dtype='uint8', norm_type='unit'):
-        self.data_path = data_path
-        self.input_shape = input_shape
-        self.scaling = scaling
-        self.len_epoch = len_epoch
-        self.sampling_mode = sampling_mode
-        self.in_channels = in_channels
-        self.orientations = orientations
-        self.orientation = 0
-        self.k = 0
-        self.batch_size = batch_size
-        self.norm_type = norm_type
-
-        # load the data
-        self.data = []
-        self.data_sizes = []
-        for k, path in enumerate(data_path):
-            data = read_volume(path, type=types[k], dtype=dtype)
-
-            # rescale the dataset if necessary
-            if scaling is not None:
-                target_size = np.asarray(np.multiply(data.shape, scaling[k]), dtype=int)
-                data = \
-                    F.interpolate(torch.Tensor(data[np.newaxis, np.newaxis, ...]), size=tuple(target_size),
-                                  mode='area')[0, 0, ...].numpy()
-
-            self.data.append(data)
-            self.data_sizes.append(data.size)
-        self.data_sizes = np.array(self.data_sizes)
-        self.data_sizes = self.data_sizes / np.sum(self.data_sizes)
-
-    def __getitem__(self, i):
-        pass
-
-    def __len__(self):
-        return self.len_epoch
-
-    def _get_stats(self):
-        mu = []
-        std = []
-
-        for data in self.data:
-            mu.append(np.mean(data))
-            std.append(np.std(data))
-
-        return mu, std
-
-    def _select_orientation(self):
-        self.orientation = np.random.choice(self.orientations)
-
-    def _select_dataset(self):
-        if self.sampling_mode == 'uniform':
-            self.k = np.random.randint(0, len(self.data))
-        else:
-            self.k = np.random.choice(len(self.data), p=self.data_sizes)
